(document:surround "/std/frame")

;;; Functions

(define (read-from-backend)
  (woo-catch/message (lambda()
      (let* ((ipv (form-value "ipv"))
             (is-ipv4 (string=? ipv "4"))
             (data (woo-read-first "/net-iptables" 'ipv ipv)))
	(form-update-enum "ifaces" (woo-list "/net-iptables/ifaces" 'ipv ipv))
	(form-update-enum "services" (woo-list "/net-iptables/services" 'ipv ipv))
	(form-update-value-list '("status") data)
	(ifaces    value (woo-get-option data 'external_ifaces))
	(services  value (woo-get-option data 'opened_services))
	(tcp-ports value (woo-get-option data 'opened_tcp_ports))
	(udp-ports value (woo-get-option data 'opened_udp_ports))
	(gw visibility is-ipv4)
	(ifaces activity (not (string=? (woo-get-option data 'mode) "host")))
	(gw state (string=? (woo-get-option data 'mode) "gateway"))
	(fw state (string=? (woo-get-option data 'mode) "router"))
	(host state (string=? (woo-get-option data 'mode) "host")) ))))

(define (write-to-backend)
  (woo-catch/message (lambda()
    (woo-write "/net-iptables"
      'ipv (form-value "ipv")
      'status (form-value "status")
      'mode (if (gw state) "gateway" (if (fw state) "router" "host"))
      'external_ifaces (ifaces value)
      'opened_services (services value)
      'opened_tcp_ports (tcp-ports value)
      'opened_udp_ports (udp-ports value) ))))

(define (init-ui)
  (woo-catch/message (lambda()
    (let ((available-ip-versions (woo-list "/net-iptables/ipv" 'language (form-value "language"))))
      (form-update-enum "ipv" available-ip-versions)
      ;; Don't show IP version selector if only one IP version is available
      (form-update-visibility "ipv" (not (= (length available-ip-versions) 1))))))
  (form-update-value "ipv" "4")
  (read-from-backend))

;;; UI

(gridbox
  columns "50;50"
  margin 20

  (hbox align "left"
        (label name "ipv" text (_ "Select IP version:"))
        (combobox name "ipv")
        (checkbox name "status" text (_ "Enable firewall")))

  (spacer)
  (document:id fw_separator (separator colspan 2))
  (document:id fw_label (label text (_ "Select forwarding mode:") align "top" rowspan 3))
  (document:id fw (radio text (_ "router")))
  (document:id gw (radio text (_ "gateway")))
  (document:id host (radio text (_ "host")))

  (separator colspan 2)

  (label text (_ "Select external interfaces:"))
  (label text (_ "Allow incoming connections on external interfaces:") align "center")

  (document:id ifaces
    (checklistbox name "ifaces"))

  (gridbox columns "100"

    (label text (_ "Services:"))
    (document:id services
      (checklistbox name "services"))

    (label text (_ "Extra TCP ports:"))
    (document:id tcp-ports (edit name "tcp_ports"))
    (label text (_ "(separated by commas or spaces)"))

    (label)
    (label text (_ "Extra UDP ports:"))
    (document:id udp-ports (edit name "udp_ports"))
    (label text (_ "(separated by commas or spaces)"))
  )

  ;;
  (label)
  (hbox align "center"
    (button text (_ "Apply")
      (when clicked (write-to-backend) (read-from-backend)))
    (label "  ")
    (button text (_ "Reset")
      (when clicked (read-from-backend))) ) )


(document:root (when loaded (init-ui))
               (form-bind "ipv"    "change" read-from-backend))
