(define-module (ui net-tc ajax)
    :use-module (alterator ajax)
    :use-module (alterator woo)
    :use-module (alterator algo)
    :export (init))

(define (read-from-backend)
  (let ((name (or (form-value "name") "default")))
    (catch/message (lambda ()
       (form-update-enum "services"
               (woo-list "/net-iptables/services" 'ipv (form-value "ipv") 'language (form-value "language")))
      (form-update-enum "name" (woo-list "/net-tc" 'ipv (form-value "ipv") 'language (form-value "language")))
      (form-update-value-list
        '("status" "ifaces" "mac" "ikbps" "okbps" "services" "tcp_ports" "udp_ports" "commit_mode" "comm")
        (woo-read-first "/net-tc"  'ipv (form-value "ipv") 'name name 'language (form-value "language") )) ))
    (form-update-value "ip"   name)
    (form-update-value "name" name)
    (form-update-visibility "manmode_warn" (not (form-value "commit_mode")))
    (let ((ndef (not (string=? "default" name)))
          (ipv4 (string=? "4" (form-value "ipv"))))
      (form-update-value "allowed_conn_label"
        (if ndef
          (_ "Additional allowed connections:" "alterator-net-iptables")
          (_ "Allowed connections for all authorized clients:" "alterator-net-iptables") ))
      (form-update-visibility "del" ndef)
      (form-update-visibility "ip" ndef)
      (form-update-visibility "tc" (and ndef ipv4))
      (form-update-visibility "ikbps" (and ndef ipv4))
      (form-update-visibility "okbps" (and ndef ipv4))
      (form-update-visibility "resolve" ndef)
      (form-update-visibility "mac" ndef)
      (form-update-visibility "comm" ndef) )))

(define (ipv-changed)
  (form-update-value "name" #f)
  (read-from-backend))

(define (switch)
  (catch/message (lambda ()
    (woo "switch" "/net-tc" 'ipv (form-value "ipv") 'status (form-value "status")))))

(define (resolve)
  (catch/message (lambda ()
    (form-update-value "mac"
      (woo-get-option
        (cond-car
          (woo "resolve" "/net-tc" 'name (form-value "name") 'ipv (form-value "ipv")))
         'mac "")))))

(define (add)
  (catch/message (lambda ()
    (woo "add" "/net-tc" 'new_ip (form-value "new_ip") 'ipv (form-value "ipv") )))
  (form-update-value "name" (form-value "new_ip"))
  (form-update-value "new_ip" "")
  (read-from-backend) )

(define (del)
  (catch/message (lambda ()
    (woo "del" "/net-tc" 'name (form-value "name") 'ipv (form-value "ipv"))))
  (form-update-value "name" "default")
  (read-from-backend) )

(define (app)
  (catch/message (lambda ()
    (apply woo-write "/net-tc" (form-value-list
      '("ipv" "language" "services" "tcp_ports" "udp_ports" "name" "mac" "ikbps" "okbps" "comm"))) ))
  (read-from-backend) )

(define (init-ui)
  (catch/message (lambda()
    (let ((available-ip-versions (woo-list "/net-iptables/ipv" 'language (form-value "language"))))
      (form-update-enum "ipv" available-ip-versions)
      ;; Don't show IP version selector if only one IP version is available
      (form-update-visibility "area-ipv-select" (not (= (length available-ip-versions) 1))))))
  (form-update-value "ipv" "4")
  (read-from-backend))

(define (init)
  (init-ui)
  (form-bind "ipv"     "change" ipv-changed)
  (form-bind "name"    "change" read-from-backend)
  (form-bind "status"  "change" switch)
  (form-bind "resolve" "click"  resolve)
  (form-bind "add"     "click"  add)
  (form-bind "del"     "click"  del)
  (form-bind "apply"   "click"  app))
